@@ -1,25 +1,77 @@
+"""
+Generate ReST docs with figures for each model.
+
+usage: python genmodels.py path/to/model1.py ...
+
+Output is placed in the directory model, with *model1.py* producing
+*model/model1.rst* and *model/img/model1.png*.
+
+Set the environment variable SASMODELS_BUILD_CACHE to the path for the
+build cache. That way the figure will only be regenerated if the kernel
+code has changed.  This is useful on a build server where the environment
+is created from scratch each time.  Be sure to clear the cache from time
+to time so it doesn't get too large.  Also, the cache will need to be
+cleared if the image generation is updated, either because matplotib
+is upgraded or because this file changes.  To accomodate both these
+conditions set the path as the following in your build script::
+
+    SASMODELS_BUILD_CACHE=/tmp/sasbuild_$(shasum genmodel.py | cut -f 1 -d" ")
+
+Putting the cache in /tmp allows temp-reaper to clean it up automatically.
+Putting the sha1 hash of this file in the cache directory name means that
+a new cache will be created whenever the text of this file has changed,
+even if it is downloaded from a different branch of the repo.
+
+The release build should not use any caching.
+
+This program uses multiprocessing to build the jobs in parallel.  Use
+the following::
+
+    SASMODELS_BUILD_CPUS=0  # one per processor
+    SASMODELS_BUILD_CPUS=1  # don't use multiprocessing
+    SASMODELS_BUILD_CPUS=n  # use n processes
+
+Note that sasmodels with OpenCL is very good at using all the processors
+when generating the model plot, so you should only use a small amount
+of parallelism (maybe 2-4 processes), allowing matplotlib to run in
+parallel.  More parallelism won't help, and may overwhelm the GPU if you
+have one.
+"""
 from __future__ import print_function
 
-import sys, os, math, re
+import sys
+import os
+from os.path import basename, dirname, realpath, join as joinpath, exists
+import math
+import re
+import shutil
+
+# CRUFT: python 2.7 backport of makedirs(path, exist_ok=False)
+if sys.version_info[0] >= 3:
+    from os import makedirs
+else:
+    def makedirs(path, exist_ok=False):
+        try:
+            os.makedirs(path)
+        except Exception:
+            if not exist_ok or not exists(path):
+                raise
+
 import numpy as np
-import matplotlib
-matplotlib.use('Agg')
-import matplotlib.pyplot as plt
-sys.path.insert(0, os.path.abspath('..'))
-import sasmodels
+
+# TODO: Remove this line when genmodel is moved to the sasmodels directory.
+sys.path.insert(0, realpath(joinpath(dirname(__file__), '..')))
 from sasmodels import generate, core
 from sasmodels.direct_model import DirectModel, call_profile
 from sasmodels.data import empty_data1D, empty_data2D
 
 try:
     from typing import Dict, Any
-except ImportError:
-    pass
-else:
-    from matplotlib.axes import Axes
+    #from matplotlib.axes import Axes
     from sasmodels.kernel import KernelModel
     from sasmodels.modelinfo import ModelInfo
-
+except ImportError:
+    pass
 
 def plot_1d(model, opts, ax):
     # type: (KernelModel, Dict[str, Any], Axes) -> None
@@ -60,6 +112,11 @@ def plot_2d(model, opts, ax):
     ax.set_ylabel(r'$Q_y \/(\AA^{-1})$')
 
 def plot_profile_inset(model_info, ax):
+    # type: (ModelInfo, Axes) -> None
+    """
+    Plot 1D radial profile as inset plot.
+    """
+    import matplotlib.pyplot as plt
     p = ax.get_position()
     width, height = 0.4*(p.x1-p.x0), 0.4*(p.y1-p.y0)
     left, bottom = p.x1-width, p.y1-height
@@ -83,6 +140,8 @@ def make_figure(model_info, opts):
     """
     Generate the figure file to include in the docs.
     """
+    import matplotlib.pyplot as plt
+
     model = core.build_model(model_info)
 
     fig_height = 3.0 # in
@@ -123,23 +182,29 @@ def make_figure(model_info, opts):
         plot_profile_inset(model_info, ax1d)
 
     # Save image in model/img
-    path = os.path.join('model', 'img', figfile(model_info))
+    makedirs(joinpath('model', 'img'), exist_ok=True)
+    path = joinpath('model', 'img', figfile(model_info))
     plt.savefig(path, bbox_inches='tight')
+    plt.close(fig)
     #print("figure saved in",path)
 
+def newer(src, dst):
+    return not exists(dst) or os.path.getmtime(src) > os.path.getmtime(dst)
+
 def copy_if_newer(src, dst):
-    from os.path import dirname, exists, getmtime
-    import shutil
-    if not exists(dst):
-        path = dirname(dst)
-        if not exists(path):
-            os.makedirs(path)
-        shutil.copy2(src, dst)
-    elif getmtime(src) > getmtime(dst):
+    # type: (str) -> str
+    """
+    Copy from *src* to *dst* if *src* is newer or *dst* doesn't exist.
+    """
+    if newer(src, dst):
+        makedirs(dirname(dst), exist_ok=True)
         shutil.copy2(src, dst)
 
 def link_sources(model_info):
-    from os.path import basename, dirname, realpath, join as joinpath
+    # type: (ModelInfo) -> str
+    """
+    Add link to model sources from the doc tree.
+    """
 
     # List source files in reverse order of dependency.
     model_file = basename(model_info.filename)
@@ -174,9 +239,9 @@ def link_sources(model_info):
     # Probably need to dump all the rst files into an index.rst to build them.
 
     # Link to github repo (either the tagged sasmodels version or master)
-    url = "https://github.com/SasView/sasmodels/blob/v%s"%sasmodels.__version__
+    #url = "https://github.com/SasView/sasmodels/blob/v%s"%sasmodels.__version__
     #url = "https://github.com/SasView/sasmodels/blob/master"%sasmodels.__version__
-    links = ["`%s <%s/sasmodels/models/%s>`_"%(path, url, path) for path in sources]
+    #links = ["`%s <%s/sasmodels/models/%s>`_"%(path, url, path) for path in sources]
 
     sep = "\n$\\ \\star\\ $ "  # bullet
     body = "\n**Source**\n"
@@ -184,12 +249,11 @@ def link_sources(model_info):
     body += "\n" + sep.join(downloads) + "\n\n"
     return body
 
-def gen_docs(model_info):
-    # type: (ModelInfo) -> None
+def gen_docs(model_info, outfile):
+    # type: (ModelInfo, str) -> None
     """
     Generate the doc string with the figure inserted before the references.
     """
-
     # Load the doc string from the module definition file and store it in rst
     docstr = generate.make_doc(model_info)
 
@@ -204,7 +268,7 @@ def gen_docs(model_info):
     captionstr += '\n'
 
     # Add figure reference and caption to documentation (at end, before References)
-    pattern = '\*\*REFERENCE'
+    pattern = r'\*\*REFERENCE'
     match = re.search(pattern, docstr.upper())
 
     sources = link_sources(model_info)
@@ -221,20 +285,87 @@ def gen_docs(model_info):
         print('------------------------------------------------------------------')
         docstr += insertion
 
-    open(sys.argv[2], 'w').write(docstr)
+    with open(outfile, 'w') as fid:
+        fid.write(docstr)
 
-def process_model(path):
+def make_figure_cached(model_info, opts):
+    """
+    Cache sasmodels figures between independent builds.
+
+    To enable caching, set *SASMODELS_BUILD_CACHE* in the environment.
+    A (mostly) unique key will be created based on model source and opts.  If
+    the png file matching that key exists in the cache it will be copied into
+    the documentation tree, otherwise a new png file will be created and copied
+    into the cache.
+
+    Be sure to clear the cache from time to time.  Even though the model
+    source
+    """
+    import hashlib
+
+    # check if we are caching
+    cache_dir = os.environ.get('SASMODELS_BUILD_CACHE', None)
+    if cache_dir is None:
+        make_figure(model_info, opts)
+        return
+
+    # TODO: changing default parameters won't trigger a rebuild.
+    # build cache identifier
+    if callable(model_info.Iq):
+        with open(model_info.filename) as fid:
+            source = fid.read()
+    else:
+        source = generate.make_source(model_info)["dll"]
+    pars = str(sorted(model_info.parameters.defaults.items()))
+    code = source + pars + str(sorted(opts.items()))
+    hash_id = hashlib.sha1(code.encode('utf-8')).hexdigest()
+
+    # copy from cache or generate and copy to cache
+    png_name = figfile(model_info)
+    target_fig = joinpath('model', 'img', png_name)
+    cache_fig = joinpath(cache_dir, ".".join((png_name, hash_id, "png")))
+    if exists(cache_fig):
+        copy_file(cache_fig, target_fig)
+    else:
+        #print(f"==>regenerating png {model_info.id}")
+        make_figure(model_info, opts)
+        copy_file(target_fig, cache_fig)
+
+def copy_file(src, dst):
+    # type: (str) -> str
+    """
+    Copy from *src* to *dst*, making the destination directory if needed.
+    """
+    if not exists(dst):
+        path = dirname(dst)
+        makedirs(path, exist_ok=True)
+        shutil.copy2(src, dst)
+    elif os.path.getmtime(src) > os.path.getmtime(dst):
+        shutil.copy2(src, dst)
+
+def process_model(py_file, force=False):
     # type: (str) -> None
     """
     Generate doc file and image file for the given model definition file.
+
+    Does nothing if the corresponding rst file is newer than *py_file*.
+    Also checks the timestamp on the *genmodel.py* program (*__file__*),
+    since we want to rerun the generator on all files if we change this
+    code.
+
+    If *force* then generate the rst file regardless of time stamps.
     """
+    rst_file = joinpath('model', basename(py_file).replace('.py', '.rst'))
+    if not (force or newer(py_file, rst_file) or newer(__file__, rst_file)):
+        #print("skipping", rst_file)
+        return
 
     # Load the model file
-    model_name = os.path.basename(path)[:-3]
+    model_name = basename(py_file)[:-3]
     model_info = core.load_model_info(model_name)
 
     # Plotting ranges and options
-    opts = {
+    PLOT_OPTS = {
         'xscale'    : 'log',
         'yscale'    : 'log' if not model_info.structure_factor else 'linear',
         'zscale'    : 'log' if not model_info.structure_factor else 'linear',
@@ -250,8 +381,31 @@ def process_model(path):
     }
 
     # Generate the RST file and the figure.  Order doesn't matter.
-    gen_docs(model_info)
-    make_figure(model_info, opts)
+    print("generating", rst_file)
+    gen_docs(model_info, rst_file)
+    make_figure_cached(model_info, PLOT_OPTS)
+
+def main(cpus=None):
+    """
+    Process files listed on the command line via :func:`process_model`.
+
+    *cpus* indicates the number of parallel processes. Use *cpus=0* for one
+    process per cpu or *cpus=1* for no multiprocessing. If *cpus=None* then
+    use the value in SASMODELS_BUILD_CPUS from the environment, or 0 if no
+    environment variable is found.
+    """
+    import matplotlib
+    matplotlib.use('Agg')
+
+    if cpus is None:
+        cpus = int(os.environ.get("SASMODELS_BUILD_CPUS", "0"))
+    if cpus != 1:
+        import multiprocessing
+        p = multiprocessing.Pool(cpus if cpus > 0 else None)
+        p.map(process_model, sys.argv[1:])
+    else:
+        for py_file in sys.argv[1:]:
+            process_model(py_file)
 
 if __name__ == "__main__":
-    process_model(sys.argv[1])
+    main()
